@@ -1359,7 +1359,9 @@ class BrainProxy2:
                 if tc.get("id"):
                     accum["id"] = tc["id"]
                 
-                yield f"data: {json.dumps(payload)}\n\n"
+                # DON'T yield tool call chunks - they confuse the client
+                # The client expects text content, not tool calls
+                # yield f"data: {json.dumps(payload)}\n\n"
             
             # Store finish reason but DON'T BREAK - consume entire stream
             if choice.get("finish_reason"):
@@ -1384,9 +1386,20 @@ class BrainProxy2:
         
         self._log(f"[{stream_id}] Stream fully consumed, now executing {len(tool_call_parts)} tool calls for tenant {tenant}")
         
-        # Send a keep-alive comment to maintain SSE connection
-        yield f": Executing {len(tool_call_parts)} tool(s)...\n\n"
-        await asyncio.sleep(0.001)  # Small delay to ensure client processes the keep-alive
+        # Send a proper data event to maintain SSE connection (not just a comment)
+        # This ensures the client knows the stream is still active
+        status_chunk = {
+            "choices": [{
+                "index": 0,
+                "delta": {"content": ""},  # Empty content to keep stream alive
+                "finish_reason": None
+            }],
+            "object": "chat.completion.chunk",
+            "created": int(time.time()),
+            "model": request.model or self.config.default_model
+        }
+        yield f"data: {json.dumps(status_chunk)}\n\n"
+        await asyncio.sleep(0.001)  # Small delay to ensure client processes
         
         # Execute tool calls and stream follow-up
         tool_calls = list(tool_call_parts.values())
@@ -1448,8 +1461,18 @@ class BrainProxy2:
         
         self._log(f"[{stream_id}] Making non-streaming follow-up call after tool execution for tenant {tenant}")
         
-        # Send another keep-alive to maintain connection
-        yield f": Processing tool results...\n\n"
+        # Send another data event to maintain connection
+        processing_chunk = {
+            "choices": [{
+                "index": 0,
+                "delta": {"content": ""},  # Empty content
+                "finish_reason": None
+            }],
+            "object": "chat.completion.chunk",
+            "created": int(time.time()),
+            "model": request.model or self.config.default_model
+        }
+        yield f"data: {json.dumps(processing_chunk)}\n\n"
         
         # Make a NON-STREAMING call WITHOUT TOOLS to force text generation
         # We explicitly don't pass tools to prevent additional tool calls
